@@ -19,31 +19,37 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/signal"
+	"path/filepath"
 	"runtime"
 	"strconv"
 	"strings"
+	"syscall"
 	"time"
 
-	"mime"
+	"github.com/gohugoio/hugo/livereload"
 
+	"github.com/gohugoio/hugo/config"
+
+	"github.com/gohugoio/hugo/helpers"
 	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
-	"github.com/spf13/hugo/helpers"
-	"github.com/spf13/hugo/hugofs"
 	jww "github.com/spf13/jwalterweatherman"
-	"github.com/spf13/viper"
 )
 
 var (
 	disableLiveReload bool
+	navigateToChanged bool
 	renderToDisk      bool
 	serverAppend      bool
 	serverInterface   string
 	serverPort        int
+	liveReloadPort    int
 	serverWatch       bool
-)
+	noHTTPCache       bool
 
-//var serverCmdV *cobra.Command
+	disableFastRender bool
+)
 
 var serverCmd = &cobra.Command{
 	Use:     "server",
@@ -88,145 +94,301 @@ func init() {
 	initHugoBuilderFlags(serverCmd)
 
 	serverCmd.Flags().IntVarP(&serverPort, "port", "p", 1313, "port on which the server will listen")
+	serverCmd.Flags().IntVar(&liveReloadPort, "liveReloadPort", -1, "port for live reloading (i.e. 443 in HTTPS proxy situations)")
 	serverCmd.Flags().StringVarP(&serverInterface, "bind", "", "127.0.0.1", "interface to which the server will bind")
 	serverCmd.Flags().BoolVarP(&serverWatch, "watch", "w", true, "watch filesystem for changes and recreate as needed")
+	serverCmd.Flags().BoolVar(&noHTTPCache, "noHTTPCache", false, "prevent HTTP caching")
 	serverCmd.Flags().BoolVarP(&serverAppend, "appendPort", "", true, "append port to baseURL")
 	serverCmd.Flags().BoolVar(&disableLiveReload, "disableLiveReload", false, "watch without enabling live browser reload on rebuild")
+	serverCmd.Flags().BoolVar(&navigateToChanged, "navigateToChanged", false, "navigate to changed content file on live browser reload")
 	serverCmd.Flags().BoolVar(&renderToDisk, "renderToDisk", false, "render to Destination path (default is render to memory & serve from there)")
+	serverCmd.Flags().BoolVar(&disableFastRender, "disableFastRender", false, "enables full re-renders on changes")
+
 	serverCmd.Flags().String("memstats", "", "log memory usage to this file")
 	serverCmd.Flags().String("meminterval", "100ms", "interval to poll memory usage (requires --memstats), valid time units are \"ns\", \"us\" (or \"µs\"), \"ms\", \"s\", \"m\", \"h\".")
 
 	serverCmd.RunE = server
 
-	mime.AddExtensionType(".json", "application/json; charset=utf-8")
-	mime.AddExtensionType(".css", "text/css; charset=utf-8")
-
 }
 
 func server(cmd *cobra.Command, args []string) error {
-	if err := InitializeConfig(serverCmd); err != nil {
-		return err
-	}
-
-	if flagChanged(cmd.Flags(), "disableLiveReload") {
-		viper.Set("disableLiveReload", disableLiveReload)
-	}
-
-	if serverWatch {
-		viper.Set("watch", true)
-	}
-
-	if viper.GetBool("watch") {
-		serverWatch = true
-		watchConfig()
-	}
-
-	l, err := net.Listen("tcp", net.JoinHostPort(serverInterface, strconv.Itoa(serverPort)))
-	if err == nil {
-		l.Close()
-	} else {
-		if flagChanged(serverCmd.Flags(), "port") {
-			// port set explicitly by user -- he/she probably meant it!
-			return newSystemErrorF("Server startup failed: %s", err)
-		}
-		jww.ERROR.Println("port", serverPort, "already in use, attempting to use an available port")
-		sp, err := helpers.FindAvailablePort()
-		if err != nil {
-			return newSystemError("Unable to find alternative port to use:", err)
-		}
-		serverPort = sp.Port
-	}
-
-	viper.Set("port", serverPort)
-
-	baseURL, err = fixURL(baseURL)
-	if err != nil {
-		return err
-	}
-	viper.Set("baseURL", baseURL)
-
-	if err := memStats(); err != nil {
-		jww.ERROR.Println("memstats error:", err)
-	}
-
 	// If a Destination is provided via flag write to disk
 	if destination != "" {
 		renderToDisk = true
 	}
 
-	// Hugo writes the output to memory instead of the disk
-	if !renderToDisk {
-		hugofs.SetDestination(new(afero.MemMapFs))
-		// Rendering to memoryFS, publish to Root regardless of publishDir.
-		viper.Set("publishDir", "/")
+	cfgInit := func(c *commandeer) error {
+		c.Set("renderToMemory", !renderToDisk)
+		if cmd.Flags().Changed("navigateToChanged") {
+			c.Set("navigateToChanged", navigateToChanged)
+		}
+		if cmd.Flags().Changed("disableLiveReload") {
+			c.Set("disableLiveReload", disableLiveReload)
+		}
+		if cmd.Flags().Changed("disableFastRender") {
+			c.Set("disableFastRender", disableFastRender)
+		}
+		if serverWatch {
+			c.Set("watch", true)
+		}
+
+		serverPorts := make([]int, 1)
+
+		if c.languages.IsMultihost() {
+			if !serverAppend {
+				return newSystemError("--appendPort=false not supported when in multihost mode")
+			}
+			serverPorts = make([]int, len(c.languages))
+		}
+
+		currentServerPort := serverPort
+
+		for i := 0; i < len(serverPorts); i++ {
+			l, err := net.Listen("tcp", net.JoinHostPort(serverInterface, strconv.Itoa(currentServerPort)))
+			if err == nil {
+				l.Close()
+				serverPorts[i] = currentServerPort
+			} else {
+				if i == 0 && serverCmd.Flags().Changed("port") {
+					// port set explicitly by user -- he/she probably meant it!
+					return newSystemErrorF("Server startup failed: %s", err)
+				}
+				jww.ERROR.Println("port", serverPort, "already in use, attempting to use an available port")
+				sp, err := helpers.FindAvailablePort()
+				if err != nil {
+					return newSystemError("Unable to find alternative port to use:", err)
+				}
+				serverPorts[i] = sp.Port
+			}
+
+			currentServerPort = serverPorts[i] + 1
+		}
+
+		c.serverPorts = serverPorts
+
+		c.Set("port", serverPort)
+		if liveReloadPort != -1 {
+			c.Set("liveReloadPort", liveReloadPort)
+		} else {
+			c.Set("liveReloadPort", serverPorts[0])
+		}
+
+		isMultiHost := c.languages.IsMultihost()
+		for i, language := range c.languages {
+			var serverPort int
+			if isMultiHost {
+				serverPort = serverPorts[i]
+			} else {
+				serverPort = serverPorts[0]
+			}
+
+			baseURL, err := fixURL(language, baseURL, serverPort)
+			if err != nil {
+				return err
+			}
+			if isMultiHost {
+				language.Set("baseURL", baseURL)
+			}
+			if i == 0 {
+				c.Set("baseURL", baseURL)
+			}
+		}
+
+		return nil
+
 	}
 
-	if err := build(serverWatch); err != nil {
+	if err := memStats(); err != nil {
+		jww.ERROR.Println("memstats error:", err)
+	}
+
+	c, err := InitializeConfig(true, cfgInit, serverCmd)
+	if err != nil {
 		return err
 	}
 
+	if err := c.serverBuild(); err != nil {
+		return err
+	}
+
+	for _, s := range Hugo.Sites {
+		s.RegisterMediaTypes()
+	}
+
+	if serverWatch {
+		c.watchConfig()
+	}
+
 	// Watch runs its own server as part of the routine
 	if serverWatch {
-		watchDirs := getDirList()
-		baseWatchDir := viper.GetString("workingDir")
-		for i, dir := range watchDirs {
-			watchDirs[i], _ = helpers.GetRelativePath(dir, baseWatchDir)
+
+		watchDirs, err := c.getDirList()
+		if err != nil {
+			return err
 		}
 
-		rootWatchDirs := strings.Join(helpers.UniqueStrings(helpers.ExtractRootPaths(watchDirs)), ",")
+		baseWatchDir := c.Cfg.GetString("workingDir")
+		relWatchDirs := make([]string, len(watchDirs))
+		for i, dir := range watchDirs {
+			relWatchDirs[i], _ = helpers.GetRelativePath(dir, baseWatchDir)
+		}
+
+		rootWatchDirs := strings.Join(helpers.UniqueStrings(helpers.ExtractRootPaths(relWatchDirs)), ",")
 
 		jww.FEEDBACK.Printf("Watching for changes in %s%s{%s}\n", baseWatchDir, helpers.FilePathSeparator, rootWatchDirs)
-		err := NewWatcher(serverPort)
+		watcher, err := c.newWatcher(watchDirs...)
 
 		if err != nil {
 			return err
 		}
+
+		defer watcher.Close()
+
 	}
 
-	serve(serverPort)
+	return c.serve()
+
+}
+
+type fileServer struct {
+	baseURLs []string
+	roots    []string
+	c        *commandeer
+}
+
+func (f *fileServer) createEndpoint(i int) (*http.ServeMux, string, string, error) {
+	baseURL := f.baseURLs[i]
+	root := f.roots[i]
+	port := f.c.serverPorts[i]
+
+	publishDir := f.c.Cfg.GetString("publishDir")
+
+	if root != "" {
+		publishDir = filepath.Join(publishDir, root)
+	}
+
+	absPublishDir := f.c.PathSpec().AbsPathify(publishDir)
+
+	if i == 0 {
+		if renderToDisk {
+			jww.FEEDBACK.Println("Serving pages from " + absPublishDir)
+		} else {
+			jww.FEEDBACK.Println("Serving pages from memory")
+		}
+	}
+
+	httpFs := afero.NewHttpFs(f.c.Fs.Destination)
+	fs := filesOnlyFs{httpFs.Dir(absPublishDir)}
+
+	doLiveReload := !buildWatch && !f.c.Cfg.GetBool("disableLiveReload")
+	fastRenderMode := doLiveReload && !f.c.Cfg.GetBool("disableFastRender")
+
+	if i == 0 && fastRenderMode {
+		jww.FEEDBACK.Println("Running in Fast Render Mode. For full rebuilds on change: hugo server --disableFastRender")
+	}
+
+	// We're only interested in the path
+	u, err := url.Parse(baseURL)
+	if err != nil {
+		return nil, "", "", fmt.Errorf("Invalid baseURL: %s", err)
+	}
+
+	decorate := func(h http.Handler) http.Handler {
+		return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			if noHTTPCache {
+				w.Header().Set("Cache-Control", "no-store, no-cache, must-revalidate, max-age=0")
+				w.Header().Set("Pragma", "no-cache")
+			}
+
+			if fastRenderMode {
+				p := r.RequestURI
+				if strings.HasSuffix(p, "/") || strings.HasSuffix(p, "html") || strings.HasSuffix(p, "htm") {
+					f.c.visitedURLs.Add(p)
+				}
+			}
+			h.ServeHTTP(w, r)
+		})
+	}
+
+	fileserver := decorate(http.FileServer(fs))
+	mu := http.NewServeMux()
+
+	if u.Path == "" || u.Path == "/" {
+		mu.Handle("/", fileserver)
+	} else {
+		mu.Handle(u.Path, http.StripPrefix(u.Path, fileserver))
+	}
+
+	endpoint := net.JoinHostPort(serverInterface, strconv.Itoa(port))
+
+	return mu, u.String(), endpoint, nil
+}
+
+func (c *commandeer) serve() error {
+
+	isMultiHost := Hugo.IsMultihost()
+
+	var (
+		baseURLs []string
+		roots    []string
+	)
+
+	if isMultiHost {
+		for _, s := range Hugo.Sites {
+			baseURLs = append(baseURLs, s.BaseURL.String())
+			roots = append(roots, s.Language.Lang)
+		}
+	} else {
+		s := Hugo.Sites[0]
+		baseURLs = []string{s.BaseURL.String()}
+		roots = []string{""}
+	}
+
+	srv := &fileServer{
+		baseURLs: baseURLs,
+		roots:    roots,
+		c:        c,
+	}
+
+	doLiveReload := !c.Cfg.GetBool("disableLiveReload")
+
+	if doLiveReload {
+		livereload.Initialize()
+	}
+
+	var sigs = make(chan os.Signal)
+	signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)
+
+	for i, _ := range baseURLs {
+		mu, serverURL, endpoint, err := srv.createEndpoint(i)
+
+		if doLiveReload {
+			mu.HandleFunc("/livereload.js", livereload.ServeJS)
+			mu.HandleFunc("/livereload", livereload.Handler)
+		}
+		jww.FEEDBACK.Printf("Web Server is available at %s (bind address %s)\n", serverURL, serverInterface)
+		go func() {
+			err = http.ListenAndServe(endpoint, mu)
+			if err != nil {
+				jww.ERROR.Printf("Error: %s\n", err.Error())
+				os.Exit(1)
+			}
+		}()
+	}
+
+	jww.FEEDBACK.Println("Press Ctrl+C to stop")
+
+	<-sigs
 
 	return nil
 }
 
-func serve(port int) {
-	if renderToDisk {
-		jww.FEEDBACK.Println("Serving pages from " + helpers.AbsPathify(viper.GetString("publishDir")))
-	} else {
-		jww.FEEDBACK.Println("Serving pages from memory")
-	}
-
-	httpFs := afero.NewHttpFs(hugofs.Destination())
-	fs := filesOnlyFs{httpFs.Dir(helpers.AbsPathify(viper.GetString("publishDir")))}
-	fileserver := http.FileServer(fs)
-
-	// We're only interested in the path
-	u, err := url.Parse(viper.GetString("baseURL"))
-	if err != nil {
-		jww.ERROR.Fatalf("Invalid baseURL: %s", err)
-	}
-	if u.Path == "" || u.Path == "/" {
-		http.Handle("/", fileserver)
-	} else {
-		http.Handle(u.Path, http.StripPrefix(u.Path, fileserver))
-	}
-
-	jww.FEEDBACK.Printf("Web Server is available at %s (bind address %s)\n", u.String(), serverInterface)
-	jww.FEEDBACK.Println("Press Ctrl+C to stop")
-
-	endpoint := net.JoinHostPort(serverInterface, strconv.Itoa(port))
-	err = http.ListenAndServe(endpoint, nil)
-	if err != nil {
-		jww.ERROR.Printf("Error: %s\n", err.Error())
-		os.Exit(1)
-	}
-}
-
 // fixURL massages the baseURL into a form needed for serving
 // all pages correctly.
-func fixURL(s string) (string, error) {
+func fixURL(cfg config.Provider, s string, port int) (string, error) {
 	useLocalhost := false
 	if s == "" {
-		s = viper.GetString("baseURL")
+		s = cfg.GetString("baseURL")
 		useLocalhost = true
 	}
 
@@ -265,7 +427,7 @@ func fixURL(s string) (string, error) {
 				return "", fmt.Errorf("Failed to split baseURL hostpost: %s", err)
 			}
 		}
-		u.Host += fmt.Sprintf(":%d", serverPort)
+		u.Host += fmt.Sprintf(":%d", port)
 	}
 
 	return u.String(), nil
