@@ -19,22 +19,24 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/gohugoio/hugo/deps"
+
+	"github.com/gohugoio/hugo/hugolib"
+
 	"fmt"
 
+	"github.com/gohugoio/hugo/hugofs"
+
+	"github.com/gohugoio/hugo/create"
+	"github.com/gohugoio/hugo/helpers"
 	"github.com/spf13/afero"
-	"github.com/spf13/hugo/create"
-	"github.com/spf13/hugo/helpers"
-	"github.com/spf13/hugo/hugofs"
 	"github.com/spf13/viper"
+	"github.com/stretchr/testify/require"
 )
 
 func TestNewContent(t *testing.T) {
-	initViper()
-
-	err := initFs()
-	if err != nil {
-		t.Fatalf("initialization error: %s", err)
-	}
+	v := viper.New()
+	initViper(v)
 
 	cases := []struct {
 		kind     string
@@ -42,21 +44,34 @@ func TestNewContent(t *testing.T) {
 		expected []string
 	}{
 		{"post", "post/sample-1.md", []string{`title = "Post Arch title"`, `test = "test1"`, "date = \"2015-01-12T19:20:04-07:00\""}},
+		{"post", "post/org-1.org", []string{`#+title: ORG-1`}},
 		{"emptydate", "post/sample-ed.md", []string{`title = "Empty Date Arch title"`, `test = "test1"`}},
-		{"stump", "stump/sample-2.md", []string{`title = "sample 2"`}},     // no archetype file
-		{"", "sample-3.md", []string{`title = "sample 3"`}},                // no archetype
-		{"product", "product/sample-4.md", []string{`title = "sample 4"`}}, // empty archetype front matter
+		{"stump", "stump/sample-2.md", []string{`title: "Sample 2"`}},      // no archetype file
+		{"", "sample-3.md", []string{`title: "Sample 3"`}},                 // no archetype
+		{"product", "product/sample-4.md", []string{`title = "SAMPLE-4"`}}, // empty archetype front matter
+		{"shortcodes", "shortcodes/go.md", []string{
+			`title = "GO"`,
+			"{{< myshortcode >}}",
+			"{{% myshortcode %}}",
+			"{{</* comment */>}}\n{{%/* comment */%}}"}}, // shortcodes
 	}
 
-	for i, c := range cases {
-		err = create.NewContent(hugofs.Source(), c.kind, c.path)
-		if err != nil {
-			t.Errorf("[%d] NewContent: %s", i, err)
+	for _, c := range cases {
+		cfg, fs := newTestCfg()
+		ps, err := helpers.NewPathSpec(fs, cfg)
+		require.NoError(t, err)
+		h, err := hugolib.NewHugoSites(deps.DepsCfg{Cfg: cfg, Fs: fs})
+		require.NoError(t, err)
+		require.NoError(t, initFs(fs))
+
+		siteFactory := func(filename string, siteUsed bool) (*hugolib.Site, error) {
+			return h.Sites[0], nil
 		}
 
-		fname := filepath.Join("content", filepath.FromSlash(c.path))
-		content := readFileFromFs(t, hugofs.Source(), fname)
+		require.NoError(t, create.NewContent(ps, siteFactory, c.kind, c.path))
 
+		fname := filepath.Join("content", filepath.FromSlash(c.path))
+		content := readFileFromFs(t, fs.Source, fname)
 		for i, v := range c.expected {
 			found := strings.Contains(content, v)
 			if !found {
@@ -66,17 +81,17 @@ func TestNewContent(t *testing.T) {
 	}
 }
 
-func initViper() {
-	viper.Reset()
-	viper.Set("metaDataFormat", "toml")
-	viper.Set("archetypeDir", "archetypes")
-	viper.Set("contentDir", "content")
-	viper.Set("themesDir", "themes")
-	viper.Set("theme", "sample")
+func initViper(v *viper.Viper) {
+	v.Set("metaDataFormat", "toml")
+	v.Set("archetypeDir", "archetypes")
+	v.Set("contentDir", "content")
+	v.Set("themesDir", "themes")
+	v.Set("layoutDir", "layouts")
+	v.Set("i18nDir", "i18n")
+	v.Set("theme", "sample")
 }
 
-func initFs() error {
-	hugofs.InitMemFs()
+func initFs(fs *hugofs.Fs) error {
 	perm := os.FileMode(0755)
 	var err error
 
@@ -87,7 +102,7 @@ func initFs() error {
 		filepath.Join("themes", "sample", "archetypes"),
 	}
 	for _, dir := range dirs {
-		err = hugofs.Source().Mkdir(dir, perm)
+		err = fs.Source.Mkdir(dir, perm)
 		if err != nil {
 			return err
 		}
@@ -103,15 +118,39 @@ func initFs() error {
 			content: "+++\ndate = \"2015-01-12T19:20:04-07:00\"\ntitle = \"Post Arch title\"\ntest = \"test1\"\n+++\n",
 		},
 		{
-			path:    filepath.Join("archetypes", "product.md"),
-			content: "+++\n+++\n",
+			path:    filepath.Join("archetypes", "post.org"),
+			content: "#+title: {{ .BaseFileName  | upper }}",
+		},
+		{
+			path: filepath.Join("archetypes", "product.md"),
+			content: `+++
+title = "{{ .BaseFileName  | upper }}"
++++`,
 		},
 		{
 			path:    filepath.Join("archetypes", "emptydate.md"),
 			content: "+++\ndate =\"\"\ntitle = \"Empty Date Arch title\"\ntest = \"test1\"\n+++\n",
 		},
+		// #3623x
+		{
+			path: filepath.Join("archetypes", "shortcodes.md"),
+			content: `+++
+title = "{{ .BaseFileName  | upper }}"
++++
+
+{{< myshortcode >}}
+
+Some text.
+
+{{% myshortcode %}}
+{{</* comment */>}}
+{{%/* comment */%}}
+
+
+`,
+		},
 	} {
-		f, err := hugofs.Source().Create(v.path)
+		f, err := fs.Source.Create(v.path)
 		if err != nil {
 			return err
 		}
@@ -144,3 +183,16 @@ func readFileFromFs(t *testing.T, fs afero.Fs, filename string) string {
 	}
 	return string(b)
 }
+
+func newTestCfg() (*viper.Viper, *hugofs.Fs) {
+
+	v := viper.New()
+	fs := hugofs.NewMem(v)
+
+	v.SetFs(fs.Source)
+
+	initViper(v)
+
+	return v, fs
+
+}
